import { Component } from "@angular/core";
import { DataService } from "./data.service";

import * as wjcCore from "@grapecity/wijmo";
import * as wjcGrid from "@grapecity/wijmo.grid";
import * as wjcGridDetail from "@grapecity/wijmo.grid.detail";
import * as wjcGridPdf from "@grapecity/wijmo.grid.pdf";
import { Observable } from "rxjs";
import { Response } from "@angular/http";

@Component({
	selector: "app-root",
	templateUrl: "app.component.html",
	styles: ["app.component.scss"],
	providers: [DataService]
})
export class AppComponent {
	categoriesObservable$: Observable<Response>;
	categories: wjcCore.CollectionView;
	products: wjcCore.CollectionView;
	productObservable$: Observable<Response>;

	// DataSvc will be passed by derived classes
	constructor(private dataSvc: DataService) {
		this.categoriesObservable$ = this.dataSvc.getCategories();
		this.categoriesObservable$.subscribe(res => {
			// parsing the date from string in JSON
			var data = JSON.parse(JSON.stringify(res.json()), (key, value) => {
				if (typeof value === "string") {
					// Parse dates saved as JSON-strings
					let m = value.match(
						/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/
					);
					if (m) {
						return new Date(
							Date.UTC(
								+m[1],
								+m[2] - 1,
								+m[3],
								+m[4],
								+m[5],
								+m[6]
							)
						);
					}
				}
				return value;
			});
			this.categories = new wjcCore.CollectionView(data);
			// this.categories.sortComparer = function(a, b) {
			// 	if(wjcCore.isDate(a) && wjcCore.isDate(b)) {
			// 		return a.valueOf() - b.valueOf();
			// 	}
			// 	return null;
			// }
			this.categories.sortDescriptions.push(
				new wjcCore.SortDescription("CategoryDate", false)
			);
		});
	}

	initGrid(grid: wjcGrid.FlexGrid) {
		var self = this;
		new wjcGridDetail.FlexGridDetailProvider(grid, {
			createDetailCell: function(row) {
				var catId = row.dataItem.CategoryID;

				// filtering is done here because the api returns all the data
				self.products = new wjcCore.CollectionView([], {
					filter: function(item, prop) {
						return item.CategoryID == catId;
					},
					sortDescriptions: ["ProductName"]
				});
				self.productObservable$ = self.dataSvc.getProducts();
				self.productObservable$.subscribe(res => {
					self.products.sourceCollection = res.json();
				});

				// First div is to show the user that data is loading
				// Second is to create the grid
				var cell = wjcCore.createElement(`<div>
					<div>Loading......</div>
					<div style="display: none"></div>
				</div>`);
				var gridChild = new wjcGrid.FlexGrid(cell.children[1], {
					columns: [
						{ header: "Name", binding: "ProductName", width: "*" },
						{
							header: "Oty/Unit",
							binding: "QuantityPerUnit",
							width: "*",
							wordWrap: true,
							multiLine: true
						},
						{ header: "Unit Price", binding: "UnitPrice" },
						{ header: "Discontinued", binding: "Discontinued" }
					],
					headersVisibility: wjcGrid.HeadersVisibility.All,
					isReadOnly: true,
					autoGenerateColumns: false,
					allowResizing: "Both" // allow resizing of rows
				});
				self.products.sourceCollectionChanged.addHandler(function(
					s,
					e
				) {
					var c0 = cell.children[0] as HTMLDivElement;
					c0.style.display = "none";
					self.products.items[0].QuantityPerUnit = `Remove/Restore Email Template
,View Document List,Edit Document,Edit Email Template
,View Email Template List,Remove/Restore Document,Create Email Template
Remove/Restore Vote Template,Edit Quorum Rule
,Edit Governor Profile
,View Vote Option List
Template List
,Create Governor Profile
,03. Notify vote,View OSEC Governors Portal,Remove/Restore Quorum Rule
Activity Log,Remove/Restore Governor Profile
Governor Actions,View Quorum Rule
Denied,04. Notify vote Govenror Mobile,04. Edit NV,View Governor Profile
Remove/Restore Implementation Template,Create Quorum Rule
,View OSEC Portal,03. Submit vote Governor Mobile`;

					if (gridChild instanceof wjcGrid.FlexGrid && gridChild.hostElement) {
						gridChild.itemsSource = self.products;
						gridChild.hostElement.style.display = "block";
						grid.autoSizeRows();
						gridChild.invalidate();
						gridChild.rows[0].size = 200; // set the size of the row
					}
				});

				return cell;
			}
		});
	}

	exportToPDF(grid: wjcGrid.FlexGrid) {
		wjcGridPdf.FlexGridPdfConverter.export(grid, "Flexgrid.pdf", {
			drawDetailRows: true,
			customCellContent: true,
			formatItem: function(args: wjcGridPdf.PdfFormatItemEventArgs) {
				if (args.panel.cellType === wjcGrid.CellType.Cell) {
					let row = args.panel.rows[args.row];

					if (row instanceof wjcGridDetail.DetailRow) {
						let detail = wjcCore.Control.getControl(
								row.detail.children[1]
							),
							r = args.contentRect;

						if (detail) {
							wjcGridPdf.FlexGridPdfConverter.drawToPosition(
								detail,
								args.canvas.document,
								new wjcCore.Point(r.left, r.top),
								r.width,
								r.height
							);
							args.cancel = true;
						}
					}
				}
			}
		});
	}
}
